package com.xhly.manageapp

import android.annotation.SuppressLint
import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.provider.Settings
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.Toast
import androidx.appcompat.content.res.AppCompatResources
import androidx.recyclerview.widget.RecyclerView
import com.hjq.toast.Toaster
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.utils.LogShow
import com.xhly.manageapp.databinding.ItemAppBinding
import com.xhly.manageapp.utils.StrategyUtils
import java.util.Calendar

class AppAdapter(var context: Context, var data: ArrayList<AppInfo>) :
    RecyclerView.Adapter<AppAdapter.AppViewHolder>() {


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AppViewHolder {
        val inflate = ItemAppBinding.inflate(LayoutInflater.from(context), parent, false)
        return AppViewHolder(inflate)
    }

    override fun getItemCount(): Int {
        return data.size
    }




    override fun onBindViewHolder(holder: AppViewHolder, position: Int) {
        val appInfo = data[position]
        holder.mBinding.itemCv.setImageDrawable(appInfo.ico)
        holder.mBinding.itemCv.setText(appInfo.name)
        holder.mBinding.itemCv.setOnClickListener {
            if (appInfo.packageName.equals(Const.CUSTOMSPEEDAPP)){
                clearMemory()
            }else if (appInfo.packageName.equals(Const.CUSTOMCLEARAPP)){
                Toaster.showLong("清理缓存")
            }else if (appInfo.packageName.equals(Const.CUSTOMROTATIONAPP)){
                Toaster.showLong("旋转木马")
            }else{
                try {
                    var instance = Calendar.getInstance()
                    instance.set(Calendar.HOUR_OF_DAY, 11)
                    instance.set(Calendar.MINUTE, 0)
                    /* context.startActivity(appInfo.intent)*/
                    val appTimeStrategy = StrategyUtils.getAppTimeStrategy(context, appInfo.packageName)
                    if (appTimeStrategy){
                        context.startActivity(appInfo.intent)
                    }else{
                        Toaster.showLong(context.getString(R.string.apptimestoptips))
                    }
                }catch (e:Exception){
                    LogShow("应用跳转问题是"+e.toString())
                }   
            }
        }
    }

    private fun clearMemory() {
        try {
            val service : ActivityManager = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
            val runningAppProcesses = service.runningAppProcesses
            runningAppProcesses.forEach {process->
                LogShow("当前运行应用名称"+process.processName+"|||"+process?.importanceReasonComponent?.packageName)
                data.filter { it.packageName.equals(process.processName) }.forEach {
                    ManageApplication.killApplicationProcess(it.packageName)
                }
            }
            Toaster.show("加速完成。")
        }catch (e:Exception){

        }
    }

    class AppViewHolder(binding: ItemAppBinding) : RecyclerView.ViewHolder(binding.root) {
        var mBinding = binding
    }

}