package com.xhly.manageapp.service.websocket

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.widget.Toast
import com.billbook.lib.downloader.Download
import com.billbook.lib.downloader.Downloader
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.eventbus.UIEvent
import com.xhly.corelib.network.RetrofitService
import com.xhly.corelib.utils.AppUtils
import com.xhly.corelib.utils.GsonUtils
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.MD5Utils
import com.xhly.corelib.utils.SharedPreferencesUtils
import com.xhly.manageapp.ManageApplication
import com.xhly.manageapp.bean.EventLog
import com.xhly.manageapp.bean.log.LogAppStartBean
import com.xhly.manageapp.bean.log.LogdOperateBean
import com.xhly.manageapp.network.log.LogService
import com.xhly.websocket.bean.WebSocketData
import com.xhly.websocket.bean.WebSocketUserBean
import com.xhly.websocket.service.SocketClient
import com.xhly.websocket.service.WebSocketHelper
import com.xhly.websocket.utils.SocketPushUtils
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.io.File
import java.util.UUID

class AppSocket : SocketClient() {
    var logService = RetrofitService.create<LogService>()
    private val serviceScope = CoroutineScope(MainScope().coroutineContext)


    val spUtils by lazy { SharedPreferencesUtils.getInstance(context) }

    companion object {
        //标记上次打开的包名，当连续打开同一包名时，不认为打开多次
        var lastPkgName = ""
    }

    override fun startSocket(context: Context?) {
        connectUrl = Const.WSURL
        /* startNotification()*/
        LogShow("已经创建")
        //downLoadFile("")
        super.startSocket(context)
    }

    override fun processTextMessage(text: String) {
        //Log.v("websocket收到的消息", text);
        SocketPushUtils.dealPushMessage(text)
    }

    override fun onConnectedSuccess() {
        super.onConnectedSuccess()
        UIEvent(Const.UPDATE_CONNECTED_SUCCESS).post()
        val webData = WebSocketData()
        webData.code = 1001
        webData.mid = UUID.randomUUID().toString()
        val currentTimeSeconds = System.currentTimeMillis() / 1000
        webData.timeunix = currentTimeSeconds
        webData.fid = "1"
        val jsMD5 =
            MD5Utils.getJsMD5(webData.code.toString() + "_" + webData.mid + "_" + webData.timeunix.toString())
        webData.sign = MD5Utils.getJsMD5(jsMD5)
        val socketUserBean = WebSocketUserBean()
        socketUserBean.usertype = 1
        socketUserBean.username = "测试"
        socketUserBean.userid = ManageApplication.getDeviceInfo()
        webData.body = socketUserBean
        WebSocketHelper.getInstance().sendSocketString(GsonUtils.parseClassToJson(webData))
    }



    override fun onDestroy() {
        super.onDestroy()
        serviceScope.cancel()
        /* stopForeground(Service.STOP_FOREGROUND_REMOVE)*/
        val intent = Intent()
        intent.setPackage("com.xhly.manageapp")
        intent.setAction("test")
        context.sendBroadcast(intent)
        LogShow("Service被销毁")
    }
}