package com.xhly.manageapp.ui.login.activity

import BaseActivity
import android.Manifest
import android.annotation.SuppressLint
import android.content.pm.PackageManager
import android.graphics.Color
import android.os.Build
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import androidx.activity.addCallback
import androidx.work.PeriodicWorkRequest
import androidx.work.WorkManager
import com.kongzue.dialogx.dialogs.CustomDialog
import com.kongzue.dialogx.interfaces.OnBindView
import com.tbruyelle.rxpermissions2.RxPermissions
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.eventbus.UIEvent
import com.xhly.corelib.utils.AppUtils
import com.xhly.corelib.utils.GsonUtils
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.SystemUtil
import com.xhly.manageapp.ManageApplication
import com.xhly.manageapp.R
import com.xhly.manageapp.bean.EventLog
import com.xhly.manageapp.bean.LoginBean
import com.xhly.manageapp.bean.SocketMsgBean
import com.xhly.manageapp.bean.log.LogdOperateBean
import com.xhly.manageapp.bean.strategy.StrategyBean
import com.xhly.manageapp.bean.user.UserBean
import com.xhly.manageapp.contentprovider.ZSpProvider
import com.xhly.manageapp.databinding.ActivityLoginBinding
import com.xhly.manageapp.service.websocket.AppSocket
import com.xhly.manageapp.ui.login.viewmodel.LoginViewModel
import com.xhly.manageapp.ui.main.activity.MainActivity
import com.xhly.manageapp.utils.CurrentAppSharedPreferencesUtils
import com.xhly.manageapp.utils.StrategyUtils
import com.xhly.manageapp.workmanager.TimingWorker
import java.util.concurrent.TimeUnit

class LoginActivity : BaseActivity<LoginViewModel, ActivityLoginBinding>() {
    //单独创建sharedpreference用来存储初始化的应用
    private val currentSpUtils by lazy { CurrentAppSharedPreferencesUtils.getInstance(this) }
    private var regioncode = ""
    override fun getBinding() = ActivityLoginBinding.inflate(layoutInflater)

    override fun initData() {
       viewModel.loginData.observe(this){
           if (it.xhlyLoginSucess){
               loginSucess(it)
           }else{
               it.xhlyLoginMessage?.let {
                   Toast(it)
               }
           }
       }
        startWorkManager()
    }

    @SuppressLint("CheckResult")
    override fun initView() {
        initFirstUse()
        setSoftInput(mBinding.root)
        mBinding.loginBtn.setOnClickListener {
            val rxPermissions = RxPermissions(this)
            val request = rxPermissions.request(Manifest.permission.READ_PHONE_STATE)
            request.subscribe({
                login()
            },{
                login()
            })
        }
        mBinding.codeLoginIv.setOnClickListener {
            showEditDialog()
        }
        onBackPressedDispatcher.addCallback {
            //屏蔽返回
        }
    }

    private fun initFirstUse() {
        val firstFlag = currentSpUtils.getParam(Const.FIRSTFLAG, true) as Boolean
        if (firstFlag){
            //如果第一次使用则提交登录事件，并且保存初始app
            val data = LogdOperateBean()
            ManageApplication.getDeviceInfo()?.let {
                data.sn = it
            }
            data.comm = "退出登录"
            data.doEvent = EventLog.ACCOUNTQUITEVENT
            viewModel.eventLog(data)
            val firstAppList = AppUtils.GetAppList(this)
            //保存初始应用，只保留包名，因为drawble数据存储时有问题
            val pkgNameList= arrayListOf<String>()
            firstAppList.forEach {
                pkgNameList.add(it.packageName)
            }
            currentSpUtils.saveJson(Const.FIRSTAPPLIST,pkgNameList)
        }
    }

    private fun login() {
        val account = mBinding.accountEt.text.toString()
        val code = mBinding.codeEt.text.toString()
        if (account.isNullOrBlank()) {
            Toast(getString(R.string.accountnoempty))
            return
        }
        if (code.isNullOrBlank()) {
            Toast(getString(R.string.codenoempty))
            return
        }
        val loginBean=LoginBean()
        loginBean.loginname=account
        loginBean.loginpwd=code
        loginBean.device= Build.BRAND
        loginBean.deviceModel=Build.MODEL
        loginBean.regionCode=regioncode
        loginBean.clientVersion=SystemUtil.getAppVersionName(this,packageName)
        ManageApplication.getDeviceInfo()?.let {
            loginBean.sn=it
        }
        viewModel.login(loginBean)
    }

    private fun loginSucess(userBean: UserBean){
        //提交登录事件
        val data = LogdOperateBean()
        ManageApplication.getDeviceInfo()?.let {
            data.sn = it
        }
        data.doEvent = EventLog.ACCOUNTLOGINEVENT
        viewModel.eventLog(data)
        spUtils.saveJson(Const.USERINFO,userBean)
        val parseClassToJson = GsonUtils.parseClassToJson(userBean)
        ZSpProvider.save(this,Const.USERINFO,parseClassToJson)
        startIntentActivity(MainActivity().javaClass)
        finish()
    }

    override fun onResume() {
        super.onResume()
        useTimeStrategy()
    }

    private fun showEditDialog() {
        CustomDialog
            .build()
            .setCustomView(object : OnBindView<CustomDialog>(R.layout.layout_edit_dialog) {
                override fun onBind(dialog: CustomDialog, v: View) {
                    val enterTv = v.findViewById<TextView>(R.id.dialog_enter_tv)
                    val cancelTv = v.findViewById<TextView>(R.id.dialog_cancel_tv)
                    val editText = v.findViewById<EditText>(R.id.dialog_et)
                    enterTv.setOnClickListener {
                        dialog.dismiss()
                        val text = editText.text
                        val list= arrayListOf("com.xhly.manageapp")
                        spUtils.saveJson(Const.WHITELISTAPP,list)
                        regioncode = if (text != null && text.isNotEmpty()) {
                            text.toString()
                        } else {
                            ""
                        }
                    }
                    cancelTv.setOnClickListener {
                        dialog.dismiss()
                    }
                }
            }).setCancelable(false).setMaskColor(Color.parseColor("#66000000"))
            .setAlign(CustomDialog.ALIGN.CENTER).show()
    }

    override fun onUiEvent(uiEvent: UIEvent) {
        super.onUiEvent(uiEvent)
        when (uiEvent.getEvent()) {

            Const.CODE2001.toString() -> {
                val message = uiEvent.message
                showNoticeDialog(message)
            }

            Const.CODE2002.toString() -> {
                spUtils.setParam(Const.DISABLEPAD, true)
                useTimeStrategy()
            }

            Const.CODE2003.toString() -> {
                spUtils.setParam(Const.DISABLEPAD, false)
                useTimeStrategy()
            }

            Const.CODE2004.toString() -> {

            }

            Const.CODE2005.toString() -> {
                ManageApplication.rebootDevice()
            }

            Const.CODE2006.toString() -> {
                ManageApplication.launchFactoryReset()
            }

            Const.CODE2007.toString() -> {

            }

            Const.SDINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SD卡插入"
                data.doEvent = EventLog.SDEMOUNTEDVENT
                viewModel.eventLog(data)
            }

            Const.SDUNINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SD卡拔出"
                data.doEvent = EventLog.SDEJECTEVENT
                viewModel.eventLog(data)
            }

            Const.SIMINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SIM卡安装"
                data.doEvent = EventLog.SIMEVENT
                viewModel.eventLog(data)
            }

            Const.BLUETOOTHOPEN -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "蓝牙打开"
                data.doEvent = EventLog.BLUETOOHEVENT
                viewModel.eventLog(data)
            }

            Const.USB_CONNECTED -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "usb连接"
                data.doEvent = EventLog.USBCONNECTEVENT
                viewModel.eventLog(data)
            }

            Const.USB_NOCONNECTED -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "usb断开"
                data.doEvent = EventLog.USBDISCONNECTEVENT
                viewModel.eventLog(data)
            }
        }

    }

    private fun showNoticeDialog(msg: String) {
        CustomDialog.build()
            .setCustomView(object : OnBindView<CustomDialog>(R.layout.layout_notice_dialog) {
                override fun onBind(dialog: CustomDialog, v: View) {
                    val closeBtn = v.findViewById<Button>(R.id.dialog_close_btn)
                    val titleTv = v.findViewById<TextView>(R.id.dialog_title_tv)
                    val contentTv = v.findViewById<TextView>(R.id.dialog_content_tv)
                    try {
                        val msgBean = GsonUtils.parseJsonWithGson(
                            msg, SocketMsgBean().javaClass
                        ) as SocketMsgBean
                        titleTv.text = msgBean.title
                        contentTv.text = msgBean.content
                    } catch (e: Exception) {
                        LogShow("解析失败"+e.toString())
                    }
                    closeBtn.setOnClickListener {
                        dialog.dismiss()
                    }
                }
            }).setCancelable(false).setMaskColor(Color.parseColor("#66000000"))
            .setAlign(CustomDialog.ALIGN.CENTER).show()
    }

    private fun useTimeStrategy() {
        //获得sputils中的平板锁定策略
        try {
            //如果存储的是锁定则锁定
            val param = spUtils.getParam(Const.DISABLEPAD, false) as Boolean
            if (param) {
                mBinding.loginMainLlayout.visibility = View.GONE
                mBinding.lockLlayout.visibility = View.VISIBLE
            } else {
                mBinding.loginMainLlayout.visibility = View.VISIBLE
                mBinding.lockLlayout.visibility = View.GONE
            }
        } catch (e: Exception) {
            val param = spUtils.getParam(Const.DISABLEPAD, false) as Boolean
            if (param) {
                mBinding.loginMainLlayout.visibility = View.GONE
                mBinding.lockLlayout.visibility = View.VISIBLE
            } else {
                mBinding.loginMainLlayout.visibility = View.VISIBLE
                mBinding.lockLlayout.visibility = View.GONE
            }
        }
    }

    private fun startWorkManager() {
        WorkManager.getInstance(this).cancelAllWork()
        // 周期性任务时间最少15分钟 所以除以实际周期的时间  看需要加入几个任务
        val taskList = arrayListOf(0, 5, 10)
        for (taskNo in taskList) {
            val request = PeriodicWorkRequest
                // 最少15分钟 PeriodicWorkRequest
                .Builder(TimingWorker::class.java, 15, TimeUnit.MINUTES)
                // WorkRequest 根据实际周期以及任务的编号 延迟初始化 0，5，10 分别周期行执行
                .setInitialDelay(taskNo.toLong(), TimeUnit.MINUTES).build()
            WorkManager.getInstance(this).enqueue(request)
        }
    }
}