package com.xhly.manageapp.ui.main.activity

import BaseActivity
import android.Manifest
import android.annotation.SuppressLint
import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.graphics.Color
import android.graphics.drawable.Drawable
import android.provider.Settings
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.activity.addCallback
import androidx.lifecycle.viewModelScope
import androidx.recyclerview.widget.GridLayoutManager
import androidx.work.PeriodicWorkRequest
import androidx.work.WorkManager
import com.bumptech.glide.Glide
import com.bumptech.glide.request.target.CustomViewTarget
import com.bumptech.glide.request.transition.Transition
import com.google.gson.reflect.TypeToken
import com.kongzue.dialogx.dialogs.CustomDialog
import com.kongzue.dialogx.interfaces.OnBindView
import com.tbruyelle.rxpermissions2.RxPermissions
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.eventbus.UIEvent
import com.xhly.corelib.utils.AppUtils
import com.xhly.corelib.utils.CustomOSUtils
import com.xhly.corelib.utils.FileUtils
import com.xhly.corelib.utils.GsonUtils
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.SharedPreferencesUtils
import com.xhly.corelib.utils.SystemUtil
import com.xhly.manageapp.AppAdapter
import com.xhly.manageapp.ManageApplication
import com.xhly.manageapp.R
import com.xhly.manageapp.bean.EventLog
import com.xhly.manageapp.bean.SocketMsgBean
import com.xhly.manageapp.bean.UpdateBean
import com.xhly.manageapp.bean.log.LogdOperateBean
import com.xhly.manageapp.bean.school.SchoolDeviceSetBean
import com.xhly.manageapp.bean.strategy.AppTimeManageBean
import com.xhly.manageapp.bean.strategy.StrategyBean
import com.xhly.manageapp.bean.user.UserBean
import com.xhly.manageapp.broadcastreceiver.AppInstallReceiver
import com.xhly.manageapp.databinding.ActivityMainBinding
import com.xhly.manageapp.service.ManageAccessibilityService
import com.xhly.manageapp.service.websocket.AppSocket
import com.xhly.manageapp.ui.ManageActivity
import com.xhly.manageapp.ui.login.activity.LoginActivity
import com.xhly.manageapp.ui.main.viewmodel.MainViewModel
import com.xhly.manageapp.utils.AppDownLoadUtils
import com.xhly.manageapp.utils.CurrentAppSharedPreferencesUtils
import com.xhly.manageapp.utils.CustomAppUtils
import com.xhly.manageapp.utils.PadInfoUtils
import com.xhly.manageapp.utils.StrategyUtils
import com.xhly.manageapp.workmanager.TimingWorker
import kotlinx.coroutines.launch
import java.util.Calendar
import java.util.Timer
import java.util.TimerTask
import java.util.concurrent.TimeUnit


class MainActivity : BaseActivity<MainViewModel, ActivityMainBinding>() {
    var adapter: AppAdapter? = null

    private var mMyInstallReceiver: AppInstallReceiver? = null
    private var dataList: ArrayList<AppInfo> = arrayListOf()
    private var userBean: UserBean? = null
    private var timer: Timer?=null
    private val currentSpUtils by lazy {  CurrentAppSharedPreferencesUtils.getInstance(this) }
    override fun getBinding() = ActivityMainBinding.inflate(layoutInflater)

    override fun initData() {
        val json = spUtils.getFromJson(Const.USERINFO, UserBean().javaClass)
        if (json == null) {
            loginOut()
            return
        } else {
            userBean = json as UserBean
        }
        adapter = AppAdapter(this, dataList)
        mBinding.rv.layoutManager = GridLayoutManager(this, 6, GridLayoutManager.VERTICAL, false)
        mBinding.rv.adapter = adapter
        updateAppRv()
        // checkAccessibility(this)
        mBinding.btn.setOnClickListener {
            startIntentActivity(ManageActivity().javaClass)
        }
        registerAppInstallReceiver()/*
         使用情况权限
         val permissionIntent = Intent(Settings.ACTION_USAGE_ACCESS_SETTINGS)
          startActivity(permissionIntent)*/
        // csdkManagerUtils.csdkManager.enableAccessibility("com.xhly.manageapp","com.xhly.manageapp.service.TestService",true)

        viewModel.listAppData.observe(this) { modelList ->
            //获得应用集合，必装应用和推荐应用，再加上预装应用，其余应用需要静默卸载,必装应用需要进行静默下载和安装。安装应用从forceapp接口获取
            val appList = AppUtils.GetAppList(this)
            val firstAppList= arrayListOf<String>()
            try {
                //获得预装应用的包名
                val currentList =
                    currentSpUtils.getFromJson(Const.FIRSTAPPLIST, ArrayList<String>().javaClass) as ArrayList<String>
                firstAppList.addAll(currentList)
            }catch (e:Exception){
               LogShow("冻结失败"+e.toString())
            }
            appList.forEach { app ->
                val filter = modelList.filter { it.appPackage.equals(app.packageName) }
                val appInfos = firstAppList.filter { app.packageName.equals(it) }
                //接口列表以及初始化列表中不存在则将应用冻结或者卸载,目前是冻结
                LogShow("${app.name},被冻结"+(filter.isEmpty() && appInfos.isEmpty()))
                if (filter.isEmpty() && appInfos.isEmpty()){
                   ManageApplication.setPackageEnabled(app.packageName, (filter.isNotEmpty()))
                }
            }
        }
        viewModel.forceListAppData.observe(this) { modelList ->
            //获得必装应用，未安装或者版本较低则更新，先删除目录中的文件
            val appList = AppUtils.GetAppList(this)
            getExternalFilesDir(null)?.let {
                FileUtils.deleteCurrentFlie(it)
            }
            modelList.forEach { app ->
                //获得已安装的应用判断版本，未安装的则直接静默安装。
                val filter = appList.filter { it.packageName.equals(app.appPackage) }
                //列表中不存在,或者版本较低则更新
                val apkPath = app.apkPath
                val appName = app.appName
                if (filter.isNotEmpty()) {
                    val currentApp = filter[0]
                    if (app.versionNum > currentApp.versionNum) {
                        //版本低则更新安装。
                        viewModel.viewModelScope.launch {
                            AppDownLoadUtils.addToDownloadQueue(apkPath, appName)
                        }
                    }
                } else {
                    //不存在，直接下载安装。
                    viewModel.viewModelScope.launch {
                        AppDownLoadUtils.addToDownloadQueue(apkPath, appName)
                    }
                }
            }
        }

        viewModel.strategyData.observe(this) {
            spUtils.saveJson(Const.STRATEGYBEANKEY, it)
            StrategyUtils.setStrategy(it)
            useTimeStrategy()
        }
        viewModel.appStrategyData.observe(this) {
            //存储app时间策略
            spUtils.saveJson(Const.APPSTRATEGYBEANKEY, it)
        }
        viewModel.padInfoData.observe(this) {
            //存储信息上传时间
            if (it) {
                spUtils.setParam(Const.PADINFOTIMEKEY, System.currentTimeMillis())
            }
        }
        viewModel.loginOutData.observe(this) {
            if (it) {
                loginOut()
            }
        }
        viewModel.updateInfoData.observe(this) {
            //成功则保存不在提交
            if (it) {
                spUtils.setParam(Const.PADUPDATETIMEKEY, System.currentTimeMillis())
            }
        }
        viewModel.schoolSetData.observe(this) {
            spUtils.saveJson(Const.SCHOOLSETDEVICE, it)
            initSchoolSet(it)
            updateAppRv()
        }
        viewModel.wallpaperData.observe(this){
            if (!it.hPath.isNullOrBlank()){
                Glide.with(this).load(Const.URL1+it.hPath).placeholder(R.drawable.main_bg).error(R.drawable.main_bg).into(object:CustomViewTarget<LinearLayout,Drawable>(mBinding.root){
                    override fun onLoadFailed(errorDrawable: Drawable?) {
                        errorDrawable?.let {drawble->
                            mBinding.root.background=drawble
                        }
                    }

                    override fun onResourceCleared(placeholder: Drawable?) {
                        placeholder?.let {drawble->
                            mBinding.root.background=drawble
                        }
                    }

                    override fun onResourceReady(
                        resource: Drawable,
                        transition: Transition<in Drawable>?
                    ) {
                        mBinding.root.background=resource
                    }
                })
            }
        }
        viewModel.netWhiteListData.observe(this){
           StrategyUtils.setNetWhiteList(it)
        }
        startWorkManager()
    }


    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
    }

    override fun initView() {
        userBean?.let {
            mBinding.usernameTv.text = it.username

        }
        mBinding.loginoutIv.setOnClickListener {
            showEditDialog()
        }
        mBinding.codeenterCv.setOnClickListener {
            //如果密码一致,则解锁
            spUtils.setParam(Const.DISABLEPAD, false)
            useTimeStrategy()
            if (mBinding.mainLlayout.visibility == View.GONE) {
                Toast(getString(R.string.apptimestoptips))
            }
        }
        onBackPressedDispatcher.addCallback {
            //屏蔽返回
            /*    viewModel.getAppList()
                viewModel.getForceAppList()*/
            userBean?.let {
                viewModel.postDevicesetDetail(it.schoolid)
            }
        }
    }


    override fun onResume() {
        super.onResume()
        viewModel.postWallpaperPad()
        viewModel.updateStrategy()
        viewModel.uploadAppStrategy()
        viewModel.getAppList()
        viewModel.postAppNetWhiteList()
        userBean?.let {
            viewModel.postDevicesetDetail(it.schoolid)
        }
        useTimeStrategy()
        updatePadInfo()
        updateMcApp()
        getLocation()
    }

    private fun updateMcApp() {
        //每天请求一次app更新接口，这个接口是用于提交版本信息统计
        try {
            var userBean = spUtils.getFromJson(Const.USERINFO, UserBean().javaClass) as UserBean
            //存储pad更新请求时间，如果为同一天则不在上传，不然继续上传
            val padTime = spUtils.getParam(Const.PADUPDATETIMEKEY, 0L) as Long
            val instance = Calendar.getInstance()
            val currentYear = instance.get(Calendar.YEAR)
            val currentDay = instance.get(Calendar.DAY_OF_YEAR)
            instance.timeInMillis = padTime
            val year = instance.get(Calendar.YEAR)
            val day = instance.get(Calendar.DAY_OF_YEAR)
            if (!(currentYear == year && currentDay == day)) {
                val updateBean = UpdateBean()
                ManageApplication.getDeviceInfo()?.let {
                    updateBean.sn = it
                }
                val packageInfo =
                    packageManager.getPackageInfo(packageName, PackageManager.GET_META_DATA)
                updateBean.cversion = packageInfo.versionName
                updateBean.schoolid = userBean.schoolid
                updateBean.userid = userBean.userid
                updateBean.regionid = userBean.regionid
                viewModel.updateMcApp(updateBean)
            }
        } catch (e: Exception) {

        }
    }

    @SuppressLint("CheckResult")
    private fun updatePadInfo() {
        try {
            //获得pad信息提交时间，如果为同一天则不在上传，不然继续上传
            val padTime = spUtils.getParam(Const.PADINFOTIMEKEY, 0L) as Long
            val instance = Calendar.getInstance()
            val currentYear = instance.get(Calendar.YEAR)
            val currentDay = instance.get(Calendar.DAY_OF_YEAR)
            instance.timeInMillis = padTime
            val year = instance.get(Calendar.YEAR)
            val day = instance.get(Calendar.DAY_OF_YEAR)
            val rxPermissions = RxPermissions(this)
            val request = rxPermissions.request(Manifest.permission.READ_PHONE_STATE)
            //权限通过不通过都要执行
            request.subscribe({ granted: Boolean ->
                var iccid = ""
                if (granted) {
                    iccid = SystemUtil.getICCID(this)
                }
                if (!(currentYear == year && currentDay == day)) {
                    viewModel.updatePadInfo(PadInfoUtils().getPadInfo(this, iccid))
                }
            }, {
                if (!(currentYear == year && currentDay == day)) {
                    viewModel.updatePadInfo(PadInfoUtils().getPadInfo(this, ""))
                }
            })
        } catch (e: Exception) {

        }
    }

    @SuppressLint("CheckResult")
    private fun getLocation() {
        val rxPermissions = RxPermissions(this)
        rxPermissions.request(
            Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION
        ).subscribe({
            if (it) {
                SystemUtil.getLocation(this)
            } else {
                LogShow("没有权限")
            }
        }, {
            LogShow("没有权限" + it.message)
        })
    }

    private fun useTimeStrategy() {
        //获得sputils中的平板锁定策略
        try {
            //如果存储的是锁定则锁定
            var block = false
            val strategyBean =
                spUtils.getFromJson(Const.STRATEGYBEANKEY, StrategyBean::class.java) as StrategyBean
            strategyBean?.let { bean ->
                //获得是否在使用时间范围
                block = StrategyUtils.getPadTimeStrategy(bean)
            }
            if (Const.isDebug) {
                //测试状态不锁定
                block = false
            }
            val param = spUtils.getParam(Const.DISABLEPAD, false) as Boolean
            if (param || block) {
                mBinding.mainLlayout.visibility = View.GONE
                mBinding.lockLlayout.visibility = View.VISIBLE
            } else {
                mBinding.mainLlayout.visibility = View.VISIBLE
                mBinding.lockLlayout.visibility = View.GONE
            }
            changeWhiteList()
        } catch (e: Exception) {
            val param = spUtils.getParam(Const.DISABLEPAD, false) as Boolean
            if (param) {
                mBinding.mainLlayout.visibility = View.GONE
                mBinding.lockLlayout.visibility = View.VISIBLE
            } else {
                mBinding.mainLlayout.visibility = View.VISIBLE
                mBinding.lockLlayout.visibility = View.GONE
            }
        }
    }

    private fun changeWhiteList() {/*        //获得当前白名单
                val packageWhiteList = csdkManager.installPackageWhiteList
                LogShow("当前白名单$packageWhiteList")
                //获得从网络获取的白名单
                val jsonArray = spUtils.getFromJson(Const.WHITELISTAPP, ArrayList<String>().javaClass)
                val removeList = arrayListOf<String>()
                //移除白名单外的应用，并设置新的白名单
                if (jsonArray != null) {
                    val whiteList = jsonArray as ArrayList<String>
                    packageWhiteList.forEach { pkg ->
                        val filter = whiteList.filter { it == pkg }
                        if (filter.isEmpty()) {
                            removeList.add(pkg)
                        }
                    }
                    ManageApplication.addInstallPackageWhiteList(whiteList)
                }
                ManageApplication.removeInstallPackageWhiteList(removeList)*/
    }

    private fun initSchoolSet(schoolDeviceSetBean: SchoolDeviceSetBean? = null) {
        schoolDeviceSetBean?.let {bean->
            //是否启用截屏:1是0否
            ManageApplication.enableCaptureScreen(schoolDeviceSetBean.screenshots == 1)
            if (bean.forceBind == 1) {
                //TODO 设备端如何操作
            }
            if (bean.schoolClass == 1) {
                mBinding.userschoolTv.visibility = View.VISIBLE
            } else {
                mBinding.userschoolTv.visibility = View.INVISIBLE
            }
            //设置护眼模式
            CustomOSUtils.openOrCloseNightMode(this, bean.protectedEyes == 1)
            startTimeTask(bean)
        }
    }

    private fun startTimeTask(bean: SchoolDeviceSetBean) {
        if (bean.remindDuration==1&&bean.duration>0){
            //如果设置的有时间则执行任务
            try {
                timer?.cancel()
                timer= Timer()
            }catch (e:Exception){

            }
            timer?.let {
                timer?.schedule( object : TimerTask() {
                    override fun run() {
                        // 在这里编写您的定时任务代码
                        Toast(getString(R.string.durationtips))
                        startTimeTask(bean)
                    }
                },0L,bean.duration*60*1000L)
            }
        }
        //如果设置的有时间则取消所有任务
        if (bean.remindDuration!=1){
            if (timer!=null){
                timer?.cancel()
                timer=null
            }
        }
    }


    override fun onUiEvent(uiEvent: UIEvent) {
        super.onUiEvent(uiEvent)
        when (uiEvent.getEvent()) {
            "update" -> {
                updateAppRv()
            }

            Const.CODE2001.toString() -> {
                val message = uiEvent.message
                showNoticeDialog(message)
            }

            Const.CODE2002.toString() -> {
                spUtils.setParam(Const.DISABLEPAD, true)
                useTimeStrategy()
            }

            Const.CODE2003.toString() -> {
                spUtils.setParam(Const.DISABLEPAD, false)
                useTimeStrategy()
            }

            Const.CODE2004.toString() -> {
                //TODO 更新策略是否需要通过应用接口处理应用
                //请求接口更新策略
                viewModel.uploadAppStrategy()
                viewModel.updateStrategy()
            }

            Const.CODE2005.toString() -> {
                ManageApplication.rebootDevice()
            }

            Const.CODE2006.toString() -> {
                ManageApplication.launchFactoryReset()
            }

            Const.CODE2007.toString() -> {

            }

            Const.LOGAPPSTART -> {
                val message = uiEvent.message
                if (AppSocket.lastPkgName == message) {
                    return
                }
                val filter = dataList.filter { it.packageName == message }
                if (filter.isNotEmpty()) {
                    AppSocket.lastPkgName = message
                    val data = LogdOperateBean()
                    ManageApplication.getDeviceInfo()?.let {
                        data.sn = it
                    }
                    data.appPackage = message
                    data.doEvent = EventLog.STARTAPPEVENT
                    viewModel.eventLog(data)
                }
            }

            Const.APPINSTALL -> {
                val message = uiEvent.message
                updateAppRv()
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "App安装"
                data.appPackage = message
                data.doEvent = EventLog.APPINSTALLEVENT
                viewModel.eventLog(data)
            }

            Const.APPUNINSTALL -> {
                val message = uiEvent.message
                updateAppRv()
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "App卸载"
                data.appPackage = message
                data.doEvent = EventLog.APPUNINSTALLEVENT
                viewModel.eventLog(data)
            }

            Const.SDINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SD卡插入"
                data.doEvent = EventLog.SDEMOUNTEDVENT
                viewModel.eventLog(data)
            }

            Const.SDUNINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SD卡拔出"
                data.doEvent = EventLog.SDEJECTEVENT
                viewModel.eventLog(data)
            }

            Const.SIMINSTALL -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "SIM卡安装"
                data.doEvent = EventLog.SIMEVENT
                viewModel.eventLog(data)
            }

            Const.BLUETOOTHOPEN -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "蓝牙打开"
                data.doEvent = EventLog.BLUETOOHEVENT
                viewModel.eventLog(data)
            }

            Const.USB_CONNECTED -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "usb连接"
                data.doEvent = EventLog.USBCONNECTEVENT
                viewModel.eventLog(data)
            }

            Const.USB_NOCONNECTED -> {
                val data = LogdOperateBean()
                ManageApplication.getDeviceInfo()?.let {
                    data.sn = it
                }
                data.comm = "usb断开"
                data.doEvent = EventLog.USBDISCONNECTEVENT
                viewModel.eventLog(data)
            }
        }
    }

    private fun updateAppRv() {
        try {
            var schoolDeviceSetBean: SchoolDeviceSetBean? = null
            schoolDeviceSetBean = StrategyUtils.getSchoolDeviceSetBean(this)
            dataList.clear()
            var appList = AppUtils.GetAppList(this)
            appList.addAll(CustomAppUtils().getCustomAppList(this))
            if (schoolDeviceSetBean != null) {
                appList = StrategyUtils.setSchoolSetDevice(schoolDeviceSetBean, appList)
            }
            dataList.addAll(appList)
            if (adapter == null) {
                adapter = AppAdapter(this, dataList)
                mBinding.rv.adapter = adapter
            }
            adapter?.let {
                it.notifyDataSetChanged()
            }
        } catch (e: Exception) {

        }
    }

    fun checkAccessibility(context: Context): Boolean {
        // 判断辅助功能是否开启
        if (!isServiceON(this, ManageAccessibilityService::class.java.name)) {
            // 引导至辅助功能设置页面
            context.startActivity(
                Intent(Settings.ACTION_ACCESSIBILITY_SETTINGS).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            )
            Toast.makeText(context, "请先开启 \"Activity 栈\" 的辅助功能", Toast.LENGTH_LONG).show()
            return false
        }
        return true
    }

    fun isServiceON(context: Context, className: String?): Boolean {
        val activityManager = context.getSystemService(ACTIVITY_SERVICE) as ActivityManager
        val runningServices = activityManager.getRunningServices(100)
        if (runningServices.size < 0) {
            return false
        }
        for (i in runningServices.indices) {
            val service = runningServices[i].service
            if (service.className.contains(className!!)) {
                return true
            }
        }
        return false
    }


    private fun registerAppInstallReceiver() {
        val intentFilter = IntentFilter();
        intentFilter.addAction(Intent.ACTION_PACKAGE_ADDED)
        intentFilter.addAction(Intent.ACTION_PACKAGE_REMOVED)
        intentFilter.addAction(Intent.ACTION_PACKAGE_REPLACED)
        intentFilter.addDataScheme("package")
        mMyInstallReceiver = AppInstallReceiver()
        registerReceiver(mMyInstallReceiver, intentFilter);
    }


    private fun showEditDialog() {
        CustomDialog.build()
            .setCustomView(object : OnBindView<CustomDialog>(R.layout.layout_edit_dialog) {
                override fun onBind(dialog: CustomDialog, v: View) {
                    var titleTv = v.findViewById<TextView>(R.id.dialog_title_tv)
                    val enterTv = v.findViewById<TextView>(R.id.dialog_enter_tv)
                    val cancelTv = v.findViewById<TextView>(R.id.dialog_cancel_tv)
                    val editText = v.findViewById<EditText>(R.id.dialog_et)
                    editText.hint = getString(R.string.inputcode)
                    titleTv.text = getString(R.string.exit)
                    enterTv.setOnClickListener {
                        val code = editText.text
                        if (!code.isNullOrBlank()) {
                            dialog.dismiss()
                            try {
                                val userBean = spUtils.getFromJson(
                                    Const.USERINFO, UserBean().javaClass
                                ) as UserBean
                                viewModel.loginOut(userBean.userid.toString(), code.toString())
                            } catch (e: Exception) {

                            }
                        }
                    }
                    cancelTv.setOnClickListener {
                        dialog.dismiss()
                    }
                }
            }).setCancelable(false).setMaskColor(Color.parseColor("#66000000"))
            .setAlign(CustomDialog.ALIGN.CENTER).show()
    }

    private fun showNoticeDialog(msg: String) {
        CustomDialog.build()
            .setCustomView(object : OnBindView<CustomDialog>(R.layout.layout_notice_dialog) {
                override fun onBind(dialog: CustomDialog, v: View) {
                    val closeBtn = v.findViewById<Button>(R.id.dialog_close_btn)
                    val titleTv = v.findViewById<TextView>(R.id.dialog_title_tv)
                    val contentTv = v.findViewById<TextView>(R.id.dialog_content_tv)
                    try {
                        val msgBean = GsonUtils.parseJsonWithGson(
                            msg, SocketMsgBean().javaClass
                        ) as SocketMsgBean
                        titleTv.text = msgBean.title
                        contentTv.text = msgBean.content
                    } catch (e: Exception) {
                        LogShow("解析失败"+e.toString())
                    }
                    closeBtn.setOnClickListener {
                        dialog.dismiss()
                    }
                }
            }).setCancelable(false).setMaskColor(Color.parseColor("#66000000"))
            .setAlign(CustomDialog.ALIGN.CENTER).show()
    }

    private fun loginOut() {
        val data = LogdOperateBean()
        ManageApplication.getDeviceInfo()?.let {
            data.sn = it
        }
        data.comm = "退出登录"
        data.doEvent = EventLog.ACCOUNTQUITEVENT
        viewModel.eventLog(data)
        WorkManager.getInstance(this).cancelAllWork()
        deleteSharedPreferences(SharedPreferencesUtils.SP_FILE_NAME)
        startIntentActivity(LoginActivity().javaClass)
        finish()
    }


    override fun onDestroy() {
        super.onDestroy()
        if (timer!=null){
            timer?.cancel()
            timer=null
        }
        mMyInstallReceiver?.let {
            unregisterReceiver(it)
        }
        LogShow("MainActivity被销毁")
    }


    private fun startWorkManager() {
        WorkManager.getInstance(this).cancelAllWork()
        // 周期性任务时间最少15分钟 所以除以实际周期的时间  看需要加入几个任务
        val taskList = arrayListOf(0, 5, 10)
        for (taskNo in taskList) {
            val request = PeriodicWorkRequest
                // 最少15分钟 PeriodicWorkRequest
                .Builder(TimingWorker::class.java, 15, TimeUnit.MINUTES)
                // WorkRequest 根据实际周期以及任务的编号 延迟初始化 0，5，10 分别周期行执行
                .setInitialDelay(taskNo.toLong(), TimeUnit.MINUTES).build()
            WorkManager.getInstance(this).enqueue(request)
        }
    }
}