package com.xhly.manageapp.ui.main.viewmodel

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.xhly.corelib.base.viewmodel.CommonBaseViewModel
import com.xhly.corelib.network.RetrofitService
import com.xhly.manageapp.bean.AppModel
import com.xhly.manageapp.bean.ListAppBean
import com.xhly.manageapp.bean.UpdateBean
import com.xhly.manageapp.bean.WallpaperBean
import com.xhly.manageapp.bean.log.LogdOperateBean
import com.xhly.manageapp.bean.log.PadInfoBean
import com.xhly.manageapp.bean.school.SchoolDeviceSetBean
import com.xhly.manageapp.bean.strategy.AppTimeManageBean
import com.xhly.manageapp.bean.strategy.StrategyBean
import com.xhly.manageapp.network.app.AppService
import com.xhly.manageapp.network.log.LogService
import com.xhly.manageapp.network.strategy.StrategyService
import com.xhly.manageapp.ui.login.netservice.LoginService

class MainViewModel : CommonBaseViewModel() {
    private val appService by lazy { RetrofitService.create<AppService>() }
    private val loginService by lazy { RetrofitService.create<LoginService>() }
    private val strategyService by lazy { RetrofitService.create<StrategyService>() }
    private var logService = RetrofitService.create<LogService>()
    private val listData = MutableLiveData<ArrayList<AppModel>>()
    val listAppData: LiveData<ArrayList<AppModel>> = listData
    private val forceData = MutableLiveData<ArrayList<AppModel>>()
    val forceListAppData: LiveData<ArrayList<AppModel>> = forceData

    private val strategy = MutableLiveData<StrategyBean>()
    val strategyData: LiveData<StrategyBean> = strategy
    private val appStrategy = MutableLiveData<ArrayList<AppTimeManageBean>>()
    val appStrategyData: LiveData<ArrayList<AppTimeManageBean>> = appStrategy
    private val padInfo = MutableLiveData<Boolean>()
    val padInfoData: LiveData<Boolean> = padInfo
    private val updateInfo = MutableLiveData<Boolean>()
    val updateInfoData: LiveData<Boolean> = updateInfo
    private val loginOut = MutableLiveData<Boolean>()
    val loginOutData: LiveData<Boolean> = loginOut
    private val schoolSet=MutableLiveData<SchoolDeviceSetBean>()
    val schoolSetData:LiveData<SchoolDeviceSetBean> =schoolSet
    private val wallpaper=MutableLiveData<WallpaperBean>()
    val wallpaperData:LiveData<WallpaperBean> =wallpaper
    private val netWhiteList=MutableLiveData<ArrayList<String>>()
    val netWhiteListData:LiveData<ArrayList<String>> =netWhiteList

    fun getAppList() {
        launchUI {
            val postListApp = appService.postListApp(ListAppBean())
            if (postListApp.code == 0) {
                if (postListApp.obj != null && postListApp.obj.isNotEmpty()) {
                    listData.value = postListApp.obj
                } else {
                    listData.value = arrayListOf()
                }
            } else {
                if (!postListApp.msg.isNullOrEmpty()) {
                    netMessage.value = postListApp.msg
                }
            }
            //获得必装应用
            val result= appService.postForceApp()
            if (result.code==0&&result.obj!=null&&result.obj.isNotEmpty()){
                forceData.value=result.obj
            }
        }
    }

/*    fun getForceAppList(){
        launchUI {
            val result= appService.postForceApp()
            if (result.code==0&&result.obj!=null&&result.obj.isNotEmpty()){
                forceData.value=result.obj
            }
        }
    }*/

    fun updateStrategy() {
        launchUI {
            val postDetailPad = strategyService.postDetailPad()
            if (postDetailPad.code == 0) {
                if (postDetailPad.obj != null) {
                    strategy.value = postDetailPad.obj
                }
            } else {
                if (!postDetailPad.msg.isNullOrEmpty()) {
                    netMessage.value = postDetailPad.msg
                }
            }
        }
    }

    fun eventLog(operateBean: LogdOperateBean) {
        launchUI {
            logService.postLogdoperateAdd(operateBean)
        }
    }

    fun uploadAppStrategy() {
        launchUI {
            val postAppListPad = strategyService.postAppListPad()
            if (postAppListPad.code == 0) {
                if (postAppListPad.obj != null && postAppListPad.obj.size > 0) {
                    appStrategy.value = postAppListPad.obj
                } else {
                    appStrategy.value = arrayListOf()
                }
            } else {
                if (!postAppListPad.msg.isNullOrEmpty()) {
                    netMessage.value = postAppListPad.msg
                }
            }
        }
    }

    fun updatePadInfo(padInfoBean: PadInfoBean) {
        launchUI {
            val anyResponseData = logService.postPadDetailAdd(padInfoBean)
            if (anyResponseData.code == 0) {
                padInfo.value = true
            }
        }
    }

    fun updateMcApp(updateBean: UpdateBean) {
        launchUI {
            val postClientAdd = appService.postClientAdd(updateBean)
            if (postClientAdd.code == 0) {
                updateInfo.value = true
            }
        }
    }

    fun loginOut(userid: String, loginpwd: String) {
        launchUI {
            val map = HashMap<String, Any>()
            map["userid"] = userid
            map["loginpwd"] = loginpwd
            val postLoginOut = loginService.postLoginOut(map)
            if (postLoginOut.code == 0) {
                loginOut.value = true
            } else {
                if (postLoginOut.msg != null) {
                    netMessage.value = postLoginOut.msg
                }
            }
        }
    }

    /**
     * 获得设备设置管理
     */
    fun postDevicesetDetail(schoolid:Long) {
        launchUI {
            val map = HashMap<String, Any>()
            map["schoolid"] = schoolid
            val result = strategyService.postDevicesetDetail(map)
            if (result.code == 0) {
               schoolSet.value=result.obj
            } else {
                netMessage.value = result.msg
            }
        }
    }

    /**
     * 获得壁纸
     */
    fun postWallpaperPad(){
        launchUI {
            val postWallpaperPad = appService.postWallpaperPad()
            if (postWallpaperPad.code==0){
                wallpaper.value=postWallpaperPad.obj
            }
        }
    }

    /**
     * 获得网址白名单
     */
    fun postAppNetWhiteList(){
        launchUI {
            val postAppNetWhiteList = strategyService.postAppNetWhiteList()
            if (postAppNetWhiteList.code==0){
                 val urlList= arrayListOf<String>()
                postAppNetWhiteList.obj.forEach {
                    if (it.contains(";")){
                        it.split(";").forEach {str->
                            urlList.add(str)
                        }
                    }else{
                        urlList.add(it)
                    }
                }
                  netWhiteList.value=urlList
            }
        }
    }
}