package com.xhly.manageapp.utils

import android.content.Context
import com.google.gson.reflect.TypeToken
import com.xhly.corelib.Const
import com.xhly.corelib.bean.AppInfo
import com.xhly.corelib.utils.LogShow
import com.xhly.corelib.utils.ModelNameUtils
import com.xhly.corelib.utils.SharedPreferencesUtils
import com.xhly.corelib.utils.hourToCurrentDateLong
import com.xhly.manageapp.ManageApplication
import com.xhly.manageapp.bean.school.SchoolDeviceSetBean
import com.xhly.manageapp.bean.strategy.AppTimeManageBean
import com.xhly.manageapp.bean.strategy.StrategyBean
import java.util.Calendar

object StrategyUtils {
    public fun setStrategy(strategyBean: StrategyBean) {
        //是否启用白名单
        if (strategyBean.haswifi == 1) {
            ManageApplication.enableWifiWhiteList(true)
            strategyBean.padWifis?.let {
                val list = arrayListOf<String>()
                it.forEach { s ->
                    if (!s.isNullOrBlank()) {
                        list.add(s)
                    }
                }
                ManageApplication.addWifiWhiteList(list)
            }
        } else {
            ManageApplication.enableWifiWhiteList(false)
        }
        //是否开启应用策略,不使用，通过app列表接口获取。
        /*      if (strategyBean.hasapp == 1) {
                  //TODO 应用策略可能需要应用下载安装以及卸载
                  //开启
                  strategyBean.padApps?.let { appArray ->
                      val list = arrayListOf<String>()
                      appArray.forEach {
                          list.add(it.appPackage)
                      }
                      ManageApplication.addInstallPackageWhiteList(list)
                  }
              } else {
                  //不开启应用策略
                  val installPackageWhiteList = ManageApplication.getInstallPackageWhiteList()
                  ManageApplication.removeInstallPackageWhiteList(installPackageWhiteList)
              }*/

        //是否开启时间策略，这个策略在生命周期中调用getPadTimeStrategy(strategyBean)


        //是否设备策略
        if (strategyBean.hasdevice == 1) {
            //开启
            deviceStrategy(strategyBean)
        } else {
            //不设置则默认全部打开
            deviceStrategy(strategyBean)
        }

        //违规处理,直接在事件触发时判断，不再这里处理
        /*if (strategyBean.hasviolate == 1) {

        }*/
    }

    private fun deviceStrategy(strategyBean: StrategyBean) {
        val padDevices = strategyBean.padDevices
        padDevices?.forEach {
            when (it.pdtype) {
                Const.DEVICETYPE1 -> {
                    ManageApplication.disableCamera(it.enabled == 1)
                }

                Const.DEVICETYPE2 -> {
                    ManageApplication.enableBluetooth(it.enabled == 1)
                }

                Const.DEVICETYPE3 -> {
                    ManageApplication.disallowUsbDebugging(it.enabled == 1)
                }

                Const.DEVICETYPE4 -> {
                    ManageApplication.enableWifi(it.enabled == 1)
                }

                Const.DEVICETYPE5 -> {
                    ManageApplication.setPackageEnabled(Const.ANDROIDPHONE,it.enabled==1)
                    ManageApplication.setPackageEnabled(Const.ANDROIDMMS,it.enabled==1)
                }

                Const.DEVICETYPE6 -> {
                    ManageApplication.enableData(it.enabled == 1)
                }

                Const.DEVICETYPE7 -> {
                    ManageApplication.enableLocation(it.enabled == 1)
                }

                Const.DEVICETYPE8 -> {
                    ManageApplication.disallowWifiAdvanceSettings(it.enabled != 1)
                }

                Const.DEVICETYPE9 -> {
                    ManageApplication.enableMassStorage(it.enabled == 1)
                }

                Const.DEVICETYPE10 -> {
                    ManageApplication.setCustomOTG(it.enabled == 1)
                }
            }
        }
    }

    /**
     * 时间策略处理，返回为真则锁定
     */
    public fun getPadTimeStrategy(strategyBean: StrategyBean): Boolean {
        //是否开启时间策略，返回为真则锁定
        var lockFlag = false
        val dayList = arrayListOf(2, 3, 4, 5, 6, 7, 1)
        val currentTimeMillis = System.currentTimeMillis()
        if (strategyBean.hastime == 1) {
            val calendar = Calendar.getInstance()
            //返回值 1 周日 2周一 3周二 4周三 5周四 6周五 7周六
            val dayWeek = calendar.get(Calendar.DAY_OF_WEEK)
            val padTimes = strategyBean.padTimes
            if (padTimes == null || padTimes.size == 0) {
                //开启策略，但不返回时间则表示锁定
                return true
            }
            padTimes?.forEach { padTimeManageBean ->
                val day = dayList[padTimeManageBean.week - 1]
                if (day == dayWeek) {
                    val starttime = padTimeManageBean.starttime
                    val stoptime = padTimeManageBean.stoptime
                    starttime?.let {
                        val start = starttime.hourToCurrentDateLong()
                        stoptime?.let {
                            val end = stoptime.hourToCurrentDateLong()
                            //只有有一个时段符合即为可用
                            if (currentTimeMillis > start && currentTimeMillis < end) {
                                //在使用范围不锁定
                                return false
                            } else {
                                lockFlag = true
                            }
                        }
                    }
                } else {
                    //不包含今天则锁定
                    lockFlag = true
                }
            }
            return lockFlag
        }
        return false
    }

    //处理app打开策略
    public fun getAppTimeStrategy(
        context: Context,
        pkgName: String
    ): Boolean {
        try {
            val spUtils by lazy { SharedPreferencesUtils.getInstance(context) }
            val type = object : TypeToken<List<AppTimeManageBean>>() {}.type
            val appTimeManageBeanList = spUtils.getFromJsonType(
                Const.APPSTRATEGYBEANKEY,
                ArrayList<AppTimeManageBean>().javaClass,
                type
            ) as ArrayList<AppTimeManageBean>
            //为真则可以使用，为false不可以使用,为黑名单机制，只有范围内的限制，不在范围内的不限制返回true
            var useFlag = true
            val dayList = arrayListOf(2, 3, 4, 5, 6, 7, 1)
            val currentTimeMillis = System.currentTimeMillis()
            val filter = appTimeManageBeanList.filter { it.appPackage.equals(pkgName) }
            if (filter.isNotEmpty()) {
                val calendar = Calendar.getInstance()
                //返回值 1 周日 2周一 3周二 4周三 5周四 6周五 7周六
                val dayWeek = calendar.get(Calendar.DAY_OF_WEEK)
                val appTimeManageBean = filter[0]
                val appTimes = appTimeManageBean.times
                if (appTimes == null || appTimes.size == 0) {
                    //开启策略，但不返回时间则表示锁定
                    return false
                }
                appTimes?.forEach { appTimeManageBean ->
                    val day = dayList[appTimeManageBean.week - 1]
                    if (day == dayWeek) {
                        val starttime = appTimeManageBean.starttime
                        val stoptime = appTimeManageBean.stoptime
                        starttime?.let {
                            val start = starttime.hourToCurrentDateLong()
                            stoptime?.let {
                                val end = stoptime.hourToCurrentDateLong()
                                //只有有一个时段符合即为可用
                                if (currentTimeMillis > start && currentTimeMillis < end) {
                                    //在使用范围则可用
                                    return true
                                } else {
                                    useFlag = false
                                }
                            }
                        }
                    } else {
                        //不包含今天则不可用
                        useFlag = false
                    }
                }
                return useFlag
            }
            return true
        } catch (e: Exception) {
            LogShow("问题是" + e.toString())
            return true
        }
    }

    /**
     * 处理学校设置的设备设置管理,设计隐藏的隐藏图标，
     */
    public fun setSchoolSetDevice(schoolDeviceSetBean: SchoolDeviceSetBean, appList: List<AppInfo>):ArrayList<AppInfo> {
        val dataList = arrayListOf<AppInfo>()
        appList.forEach { appinfo ->
            if (ModelNameUtils.IS_LianxX505f()){
                //启用电话短信:1是0否
                if (schoolDeviceSetBean.simcard == 0 && (appinfo.packageName.equals(Const.ANDROIDPHONE) || appinfo.packageName.equals(Const.ANDROIDMMS))) {
                    return@forEach
                }
                //日历:1是0否
                if (schoolDeviceSetBean.calendar == 0 && appinfo.packageName.equals(Const.ANDROIDCALENDAR)) {
                    return@forEach
                }
                //闹钟:1是0否
                if (schoolDeviceSetBean.alarmClock == 0 && appinfo.packageName.equals(Const.ANDROIDDESKCLOCK)) {
                    return@forEach
                }
                //相机:1是0否
                if (schoolDeviceSetBean.camera == 0 && appinfo.packageName.equals(Const.ANDROIDCAMERA)) {
                    return@forEach
                }
                //相册:1是0否
                if (schoolDeviceSetBean.gallery == 0 && appinfo.packageName.equals(Const.ANDROIDGALLERY)) {
                    return@forEach
                }
            }
            //一键加速:1是0否
            if (schoolDeviceSetBean.accelerate==0&&appinfo.packageName.equals(Const.CUSTOMSPEEDAPP)){
                return@forEach
            }
            //清理缓存:1是0否
            if (schoolDeviceSetBean.cleanup==0&&appinfo.packageName.equals(Const.CUSTOMCLEARAPP)){
                return@forEach
            }
            //自动旋转:1是0否
            if (schoolDeviceSetBean.rotateSetting==0&&appinfo.packageName.equals(Const.CUSTOMROTATIONAPP)){
                return@forEach
            }
            //修改密码
            if (schoolDeviceSetBean.changePwd==0&&appinfo.packageName.equals(Const.CUSTOMCHANGEPSD)){
                return@forEach
            }
            dataList.add(appinfo)
        }
        return dataList
    }

    //获得存储的学校设备设置
    public fun getSchoolDeviceSetBean(context: Context):SchoolDeviceSetBean?{
        val spUtils = SharedPreferencesUtils.getInstance(context)
        var schoolDeviceSetBean: SchoolDeviceSetBean? = null
        try {
            val beanStr =
                spUtils.getFromJson(Const.SCHOOLSETDEVICE, SchoolDeviceSetBean().javaClass)
            if (beanStr != null) {
                schoolDeviceSetBean = beanStr as SchoolDeviceSetBean
            }
        } catch (e: Exception) {
            //为了不影响列表显示
        }
        return schoolDeviceSetBean
    }

    /**
     * 设置网络白名单
     */
    public fun setNetWhiteList(data:ArrayList<String>){
        ManageApplication.setUrlWhiteListWrite(data)
    }
}